/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.camerauploads

import com.dropbox.android.user.DbxUser

/**
 * Class that handles all decision making around which users can enable CU and which UI elements
 * those users can see.
 */
interface CameraUploadsGatingManager {
    /**
     * @return whether the passed userState can enable CU. Will also return true if CU is currently
     * enabled.
     */
    fun canEnableCuForUser(user: DbxUser?): Boolean

    /**
     * @return whether we can show the CU status bar in the photos tab for the given user.
     * This will return true for users that are a) eligible for CU or eligible with upgrade and
     * b) won't be shown CU status UI in the Home tab.
     */
    fun shouldShowCuUIInPhotosTabForUser(user: DbxUser?): Boolean

    /**
     * @return whether we must show the photos tab for the given user because this is the only UI
     * surface area where a CU user can see CU status / upsell. Once
     * `shouldShowCu{Upsell/Enable/Status}UIInHomeForUser` returns
     * true for all business users, this function can be deleted in favor of a simple
     * user.getRole() == UserRole.PERSONAL check.
     */
    fun shouldShowPhotosTabForUser(user: DbxUser?): Boolean

    /**
     * @return whether the passed user would be able to enable CU if they upgraded / linked a
     * desktop, but are currently not eligible, so we should show upsell UI in Home.
     */
    fun shouldShowCuUpsellUIInHomeForUser(user: DbxUser?): Boolean

    /**
     * @return whether the passed user would be able to enable CU if they upgraded / linked a
     * desktop, but are currently not eligible, so we should show upsell UI.
     */
    fun shouldShowCuUpsellUIForUser(user: DbxUser?): Boolean

    /**
     * @return whether the passed user has CU disabled can currently enable CU, so we can show
     * UI prompting the user to enable CU in Home.
     */
    fun shouldShowCuEnableUIInHomeForUser(user: DbxUser?): Boolean

    /**
     * @return whether the passed user would be able to enable CU if they upgraded / linked a
     * desktop, but are currently not eligible, so we should show upsell UI in Home.
     */
    fun shouldShowCuStatusUIInHomeForUser(user: DbxUser?): Boolean
}
