/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.notifications

import android.annotation.TargetApi
import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Context
import com.dropbox.android.R
import com.dropbox.base.device.Version
import com.dropbox.base.device.Version.API_26

const val DEFAULT_CHANNEL_ID = "default_channel"
const val CAMERA_UPLOAD_CHANNEL_ID = "camera_upload_channel"
const val FILE_TRANSFER_CHANNEL_ID = "file_transfer_channel"

/**
 * Helper class to set notification channels, but only on API 26+ because the NotificationChannel
 * class is new and not in the support library
 */
internal class NotificationChannelHelper(val context: Context) {

    private var manager: NotificationManager = context.getSystemService(
            Context.NOTIFICATION_SERVICE) as NotificationManager

    /**
     * Initialize the channel helper by setting up channels
     */
    fun initChannels() {
        setupChannels()
    }

    /**
     * Recreate the channels if locale changes
     */
    fun setupLocale() = setupChannels()

    /**
     * create channels with given names. According to MVP solution, we currently only support
     * three channels for categorizing notifications on Android O:
     *          - Default
     *          - Camera Uploads
     *          - File Transfer
     */
    private fun setupChannels() {
        if (Version.gte(Version.API_26)) {
            createNotificationChannel(DEFAULT_CHANNEL_ID,
                    context.resources.getString(R.string.default_channel))
            createNotificationChannel(CAMERA_UPLOAD_CHANNEL_ID,
                    context.resources.getString(R.string.camera_upload_channel))
            createNotificationChannel(FILE_TRANSFER_CHANNEL_ID,
                    context.resources.getString(R.string.file_transfer_channel))
        }
    }

    @TargetApi(API_26)
    private fun createNotificationChannel(channelId: String, channelName: String) {
        // Also allow further customize channel's default notification behaviors, customization
        // should happen before creating the channel
        val channel = NotificationChannel(
                channelId, channelName, NotificationManager.IMPORTANCE_DEFAULT)
        // Register the channel with the system; you can't change the importance
        // or other notification behaviors after this
        manager.createNotificationChannel(channel)
    }
}
