/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.payments

import android.app.PendingIntent
import android.content.Intent
import android.content.IntentSender
import android.os.Bundle
import android.os.RemoteException
import com.android.vending.billing.IInAppBillingService
import com.dropbox.base.analytics.AnalyticsLogger

/**
 * Helper for launching purchase flow in the Play Store.
 */
class GooglePlayPurchaseHelper(val packageName: String, val analyticsLogger: AnalyticsLogger) {

    /**
     * Laucnhes the purchse flow in the Play Store. Returns a non-null failure if the launch failed.
     */
    fun launchPurchaseFlow(
        billingService: IInAppBillingService,
        starter: IntentSenderStarter,
        sku: String,
        developerPayload: String,
        requestCode: Int
    ): Failure? {
        val buyIntentBundle: Bundle
        try {
            buyIntentBundle = billingService.getBuyIntent(
                3,
                packageName,
                sku,
                InAppBillingServiceUtils.ITEM_TYPE_SUBS,
                developerPayload
            )
        } catch (e: RemoteException) {
            return Failure(InAppBillingServiceUtils.PAYMENTS_HELPER_REMOTE_EXCEPTION, e)
        }

        val response = InAppBillingServiceUtils.getResponseCodeFromBundle(
            buyIntentBundle, analyticsLogger
        )

        if (response != InAppBillingServiceUtils.BILLING_RESPONSE_RESULT_OK) {
            return Failure(response)
        }

        val pendingIntent =
            buyIntentBundle
                .getParcelable<PendingIntent>(InAppBillingServiceUtils.RESPONSE_BUY_INTENT)

        try {
            starter.startIntentSenderForResult(
                pendingIntent.intentSender, requestCode, Intent(), 0, 0, 0
            )
        } catch (e: IntentSender.SendIntentException) {
            return Failure(InAppBillingServiceUtils.PAYMENTS_HELPER_SEND_INTENT_FAILED, e)
        }

        return null
    }
}
